#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_Change2CUI=y
#AutoIt3Wrapper_Res_requestedExecutionLevel=asInvoker
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****
; Sample by Joakim Schicht
; Frontend for MSCompression.dll
; https://github.com/coderforlife/ms-compress
; Many thanks to Jeffrey Bush for this excellent library
;COMPRESSION_NONE = 0, // == COMPRESSION_FORMAT_NONE
;COMPRESSION_LZX = 1, // replaces COMPRESSION_FORMAT_DEFAULT
;COMPRESSION_LZNT1 = 2, // == COMPRESSION_FORMAT_LZNT1
;COMPRESSION_XPRESS = 3, // == COMPRESSION_FORMAT_XPRESS
;COMPRESSION_XPRESS_HUFF = 4, // == COMPRESSION_FORMAT_XPRESS_HUFF
#include <WinAPI.au3>
If $cmdline[0] <> 4 Then
	ConsoleWrite("Error: Wrong number of parameters: " & $cmdline[0] & @CRLF)
	ConsoleWrite(@CRLF)
	ConsoleWrite("Usage: " & @CRLF)
	ConsoleWrite("MSCompress.exe method compression infile outfile" & @CRLF)
	ConsoleWrite(@CRLF)
	ConsoleWrite("Example compression with Xpress Huffman:" & @CRLF)
	ConsoleWrite("MSCompress.exe -c 4 %CD%\infile.bin %CD%\outfile.bin" & @CRLF)
	ConsoleWrite(@CRLF)
	ConsoleWrite("Example decompression with Xpress:" & @CRLF)
	ConsoleWrite("MSCompress.exe -d 3 %CD%\infile.bin %CD%\outfile.bin" & @CRLF)
	Exit
EndIf
If $cmdline[1] <> "-c" AND $cmdline[1] <> "-d" Then
	ConsoleWrite("Error: Param 1 must be either -c or -d: " & $cmdline[1] & @CRLF)
	Exit
EndIf
If $cmdline[2] <> 0 AND $cmdline[2] <> 1 AND $cmdline[2] <> 2 AND $cmdline[2] <> 3 AND $cmdline[2] <> 4 Then
	ConsoleWrite("Error: Param 2 must be a valid compression method (0,1,2,3,4): " & $cmdline[2] & @CRLF)
	Exit
EndIf
If NOT FileExists($cmdline[3]) Then
	ConsoleWrite("Error: Param 3 must be a valid path to inputfile: " & $cmdline[3] & @CRLF)
	Exit
EndIf
If FileExists($cmdline[4]) Then
	ConsoleWrite("Warning: output file already exist and will be overwritten: " & $cmdline[4] & @CRLF)
	Exit
EndIf
$tFile = FileOpen($cmdline[3],16)
$rFile = FileRead($tFile)
$CompressionFormatAndEngine = $cmdline[2]
If $cmdline[1] = "-c" Then
	$comp = _Compress($CompressionFormatAndEngine,$rFile)
ElseIf $cmdline[1] = "-d" Then
	$comp = _Decompress($CompressionFormatAndEngine,$rFile)
EndIf
If @error Then
	ConsoleWrite("Error: Operation failed with: " & $comp & @CRLF)
	Exit
EndIf
$outfile = FileOpen($cmdline[4],18)
FileWrite($outfile,$comp)
FileClose($outfile)
FileClose($tFile)
If NOT FileExists($cmdline[4]) Then
	ConsoleWrite("Error: Outfile not written. Are you sure path was valid: " & $cmdline[4] & @CRLF)
	Exit
EndIf
ConsoleWrite("Done!" & @CRLF)


Func _Decompress($iCompressionFormatAndEngine,$LocalInput)

Local $tInput = DllStructCreate("byte[" & BinaryLen($LocalInput) & "]")
DllStructSetData($tInput, 1, $LocalInput)
Local $tBuffer = DllStructCreate("byte[" & 3 * DllStructGetSize($tInput) & "]")
Local $dll = DllOpen(@ScriptDir & "\MSCompression.dll")

Local $aCall = DllCall($dll, "int", "decompress", _
        "dword", $iCompressionFormatAndEngine, _
        "ptr", DllStructGetPtr($tInput), _
        "dword", DllStructGetSize($tInput), _
        "ptr", DllStructGetPtr($tBuffer), _
        "dword*", 0)

If $aCall[0] = 0 Then
	Return SetError(1, 0, "Something went wrong")
EndIf
;If _WinAPI_GetLastError() <> 0 OR $aCall[0] = 0 Then
;	Return SetError(1, 0, _WinAPI_GetLastErrorMessage())
;EndIf
Local $tBuffer = DllStructCreate("byte[" & $aCall[0] & "]")
Local $bCall = DllCall($dll, "int", "decompress", _
        "dword", $iCompressionFormatAndEngine, _
        "ptr", DllStructGetPtr($tInput), _
        "dword", DllStructGetSize($tInput), _
        "ptr", DllStructGetPtr($tBuffer), _
        "dword", DllStructGetSize($tBuffer))

If $bCall[0] = 0 Then
	Return SetError(1, 0, "Something went wrong")
;If _WinAPI_GetLastError() <> 0 Then
;	Return SetError(1, 0, _WinAPI_GetLastErrorMessage())
Else
	Return DllStructGetData($tBuffer,1)
EndIf
EndFunc

Func _Compress($iCompressionFormatAndEngine,$LocalInput)

Local $tInput = DllStructCreate("byte[" & BinaryLen($LocalInput) & "]")
DllStructSetData($tInput, 1, $LocalInput)
Local $tBuffer = DllStructCreate("byte[" & DllStructGetSize($tInput) & "]")
Local $dll = DllOpen(@ScriptDir & "\MSCompression.dll")
Local $aCall = DllCall($dll, "int", "compress", _
        "dword", $iCompressionFormatAndEngine, _
        "ptr", DllStructGetPtr($tInput), _
        "dword", DllStructGetSize($tInput), _
        "ptr", DllStructGetPtr($tBuffer), _
        "dword*", 0)
If $aCall[0] = 0 Then
	Return SetError(1, 0, "Something went wrong")
EndIf
;If _WinAPI_GetLastError() <> 0 OR $aCall[0] = 0 Then
;	Return SetError(1, 0, _WinAPI_GetLastErrorMessage())
;EndIf
Local $tBuffer = DllStructCreate("byte[" & $aCall[0] & "]")
Local $bCall = DllCall($dll, "int", "compress", _
        "dword", $iCompressionFormatAndEngine, _
        "ptr", DllStructGetPtr($tInput), _
        "dword", DllStructGetSize($tInput), _
        "ptr", DllStructGetPtr($tBuffer), _
        "dword", DllStructGetSize($tBuffer))

If $bCall[0] = 0 Then
	Return SetError(1, 0, "Something went wrong")
;If _WinAPI_GetLastError() <> 0 Then
;	Return SetError(1, 0, _WinAPI_GetLastErrorMessage())
Else
	Return DllStructGetData($tBuffer,1)
EndIf
EndFunc